#!/bin/bash
#
# Script to gather the ipks that were changed in a new firmware compared to an
# older firmware. Update the variables at the top to match your files and then
# just run the script. It should generate 4 output txt files as well as fetch
# the ipks that are to be installed.


WORKING_DIR=`pwd`

# where shall the packages end?
PACKAGES_FOLDER=$WORKING_DIR/hotfix/packages

# old and new list of packages seperated by newlines
OLD_PACKAGES=$WORKING_DIR/installed-packages_fw1.txt
NEW_PACKAGES=$WORKING_DIR/installed-packages_hf6r.txt

# names of the output files
TO_BE_REMOVED=$WORKING_DIR/to_be_removed.txt
TO_BE_ADDED=$WORKING_DIR/to_be_added.txt
REALLY_REMOVED=$WORKING_DIR/really_removed.txt
REALLY_ADDED=$WORKING_DIR/really_added.txt
ERRORS=$WORKING_DIR/errors.txt


# some temp files
TMP1=$WORKING_DIR/tmp1.txt
TMP2=$WORKING_DIR/tmp2.txt


#
# gather list of new and removed packages based on the input files
#

if [ ! -f $OLD_PACKAGES ]
then
	echo "could not find the new package list at $OLD_PACKAGES; aborting"
	exit 1;
fi
if [ ! -f $NEW_PACKAGES ]
then
	echo "could not find the new package list at $NEW_PACKAGES; aborting"
	exit 1;
fi

# list of removed packages since "OLD_PACKAGES" (means also packages that were replaced in a new version!)
diff $OLD_PACKAGES $NEW_PACKAGES | grep "<" | cut -f 2 -d " " | sort > $TO_BE_REMOVED

# list of added packages in "NEW_PACKAGES" compared to "OLD_PACKAGES" (also updated versions!)
diff $OLD_PACKAGES $NEW_PACKAGES | grep ">" | cut -f 2 -d " " | sort  > $TO_BE_ADDED

cat $TO_BE_REMOVED | cut -f 1 -d "_" | sort  > $TMP1
cat $TO_BE_ADDED | cut -f 1 -d "_" | sort  > $TMP2

diff $TMP1 $TMP2 | grep "<" | cut -f 2 -d " " | sort > $REALLY_REMOVED
diff $TMP1 $TMP2 | grep ">" | cut -f 2 -d " " | sort > $REALLY_ADDED

#cleanup temp files from "gathering package list"
rm -f $TMP1 $TMP2


#
# find out filenames for "new" packages and download the files
#

# remove existing old stamp files
rm -f $WORKING_DIR/Packages.stamps.all $WORKING_DIR/Packages.stamps.armv7a $WORKING_DIR/Packages.stamps.omap3-pandora

# get recent stamp files from the feed
wget --output-document=$WORKING_DIR/Packages.stamps.all http://www.openpandora.org/feeds/unstable/all/Packages.stamps
wget --output-document=$WORKING_DIR/Packages.stamps.armv7a http://www.openpandora.org/feeds/unstable/armv7a/Packages.stamps
wget --output-document=$WORKING_DIR/Packages.stamps.omap3-pandora http://www.openpandora.org/feeds/unstable/omap3-pandora/Packages.stamps

# convert package entries in stamp files into the correct "full" urls
cat $WORKING_DIR/Packages.stamps.all | cut -f 2 -d " " | sed -e 's;^;http://www.openpandora.org/feeds/unstable/all/;g' > $TMP1
cat $WORKING_DIR/Packages.stamps.armv7a | cut -f 2 -d " " | sed -e 's;^;http://www.openpandora.org/feeds/unstable/armv7a/;g' >> $TMP1
cat $WORKING_DIR/Packages.stamps.omap3-pandora | cut -f 2 -d " " | sed -e 's;^;http://www.openpandora.org/feeds/unstable/omap3-pandora/;g' >> $TMP1

# cleanup temp files
rm -f $WORKING_DIR/Packages.stamps.all $WORKING_DIR/Packages.stamps.armv7a $WORKING_DIR/Packages.stamps.omap3-pandora

# get the list of the relevant (aka 'new) package URLs
for i in `cat $TO_BE_ADDED`;
do
	grep "$i" $TMP1 >> $TMP2;
	if [ "$?" -ne "0" ]
	then
		echo "Could not find package '$i' in download list" >> $ERRORS
	fi
done

# seperate into 'kernel' and 'other' (every packagename that starts with 'kernel' is only relevant for the kernel space
cat $TMP2 | grep  "\/kernel" > $TMP2.kernel
cat $TMP2 | grep -v "\/kernel" > $TMP2.other


#create folders for the new packages
if [ ! -d $PACKAGES_FOLDER/kernel ]
then
	mkdir -p $PACKAGES_FOLDER/kernel
fi
if [ ! -d $PACKAGES_FOLDER/other ]
then
	mkdir -p $PACKAGES_FOLDER/other
fi

# download kernel ipks
cd $PACKAGES_FOLDER/kernel
wget --no-clobber -i $TMP2.kernel

# download other ipks
cd $PACKAGES_FOLDER/other
wget --no-clobber -i $TMP2.other

# cleanup all the remaining temp files
rm -f $TMP1 $TMP2 $TMP2.kernel $TMP2.other


#
# explain the results in some nice words
#

echo -e
echo -e
echo -e
echo "Explaination of created folders and files"
echo -e
echo "Created text files:"
echo " $TO_BE_REMOVED: files that are no longer included (at least not as this exact package version)"
echo " $TO_BE_ADDED: files that are to be installed (complete package name incl version number"
echo " $REALLY_REMOVED: package names of those packages that are no longer installed (and should be removed using opkg!)"
echo " $REALLY_ADDED: package names of packages that were added since the old version"
echo " $ERRORS: a list of packages that were not found on the given download server(s)"
echo -e
echo "Saved all ("`ls $PACKAGES_FOLDER/kernel/*ipk | wc -l`") kernel related ipks to:"
echo " $PACKAGES_FOLDER/kernel"
echo "Saved all ("`ls $PACKAGES_FOLDER/other/*ipk | wc -l`") 'other' ipks to:"
echo " $PACKAGES_FOLDER/other"
echo -e
echo "You can now copy the folder $PACKAGES_FOLDER into the place where you create the pnd file."
if [ -s $ERRORS ]
then
	echo -e
	echo "Some packages were not found. Please have a look at $ERRORS and make sure that things actually work!"
fi

